@@ -98,86 +98,107 @@ isVerboseCache <- function() {
   return(isTRUE(getOption("packrat.verbose.cache")))
 }
 
-moveInstalledPackageToCache <- function(packagePath, overwrite = TRUE) {
-
-  # ensure cache directory
-  if (!file.exists(cacheLibDir()))
-    dir.create(cacheLibDir(), recursive = TRUE)
-
-  # TODO: this logic assumes the package folder, and package name,
-  # are the same (seems to always be true in practice, but in theory
-  # they could be different?)
-  descPath <- file.path(packagePath, "DESCRIPTION")
-  if (!file.exists(descPath))
-    stop("no DESCRIPTION file at path '", descPath, "' (not an R package?)")
-
-  hash <- hash(descPath)
+# helper function to remove the package from its original location and
+# create a symlink to the cached version.
+symlinkPackageToCache <- function(packagePath, cachedPackagePath) {
   packageName <- basename(packagePath)
-  cachedPackagePath <- cacheLibDir(packageName, hash, packageName)
+  backupPackagePath <- tempfile(tmpdir = dirname(packagePath))
+  if (!file.rename(packagePath, backupPackagePath)) {
+    stop("failed to back up package directory '", packagePath, "'; cannot safely link to cache.")
+  }
+  on.exit(unlink(backupPackagePath, recursive = TRUE), add = TRUE)
+
+  if (!symlink(cachedPackagePath, packagePath)) {
+    # symlink failed; attempt to restore the backup back to its original name.
+    if (!file.rename(backupPackagePath, packagePath)) {
+      stop("failed to restore package from '", backupPackagePath, "' to ",
+           "'", packagePath, "' after symlink to ",
+           "'", cachedPackagePath, "' failed; package may be lost")
+    }
+    stop("failed to create a symlink from '", packagePath, "' to '", cachedPackagePath, "'")
+  }
+
+  if (isVerboseCache()) {
+    message("Using cached ", packageName, ".")
+  }
+  return(cachedPackagePath)
+}
+
+# Given a path to an installed package (outside the packrat cache), move that
+# package into the cache and replace the original directory with a symbolic
+# link into the package cache.
+#
+# If the package already exists inside the cache, overwrite=TRUE causes
+# replacement of the cached content while overwrite=FALSE with fatal=FALSE
+# uses the cached package. Using overwrite=TRUE with fatal=TRUE will err.
+moveInstalledPackageToCache <- function(packagePath,
+                                        hash,
+                                        overwrite = TRUE,
+                                        fatal = FALSE,
+                                        cacheDir = cacheLibDir())
+{
+  ensureDirectory(cacheDir)
+
+  packageName <- basename(packagePath)
+  cachedPackagePath <- file.path(cacheDir, packageName, hash, packageName)
   backupPackagePath <- tempfile(tmpdir = dirname(cachedPackagePath))
 
   # check for existence of package in cache
-  if (file.exists(cachedPackagePath) && !overwrite)
-    stop("cached package already exists at path '", cachedPackagePath, "'")
+  if (file.exists(cachedPackagePath)) {
+    if (fatal && !overwrite) {
+      stop("cached package already exists at path '", cachedPackagePath, "'")
+    }
+
+    if (!fatal) {
+      return(symlinkPackageToCache(packagePath, cachedPackagePath))
+    }
+  }
 
   # back up a pre-existing cached package (restore on failure)
   if (file.exists(cachedPackagePath)) {
-    if (!file.rename(cachedPackagePath, backupPackagePath))
+    if (!file.rename(cachedPackagePath, backupPackagePath)) {
       stop("failed to back up package '", packageName, "'; cannot safely copy to cache")
+    }
     on.exit(unlink(backupPackagePath, recursive = TRUE), add = TRUE)
   }
 
   if (isVerboseCache()) {
     message("Caching ", packageName, ".")
   }
-  
-  # attempt to rename to cache
-  if (suppressWarnings(file.rename(packagePath, cachedPackagePath)))
-    return(cachedPackagePath)
 
-  # attempt to copy to cache
-  if (all(dir_copy(packagePath, cachedPackagePath))) {
-    unlink(packagePath, recursive = TRUE)
-    return(cachedPackagePath)
+  # attempt to rename to cache
+  if (suppressWarnings(file.rename(packagePath, cachedPackagePath))) {
+    return(symlinkPackageToCache(packagePath, cachedPackagePath))
+  }
+
+  # rename failed; copy to temporary destination in same directory
+  # and then attempt to rename from there
+  tempPath <- tempfile(tmpdir = dirname(cachedPackagePath))
+  on.exit(unlink(tempPath, recursive = TRUE), add = TRUE)
+  if (all(dir_copy(packagePath, tempPath))) {
+
+    # check to see if the cached package path exists now; if it does,
+    # assume that this was generated by another R process that successfully
+    # populated the cache
+    if (file.exists(cachedPackagePath)) {
+      return(symlinkPackageToCache(packagePath, cachedPackagePath))
+    }
+
+    # attempt to rename to target path
+    if (suppressWarnings(file.rename(tempPath, cachedPackagePath))) {
+      return(symlinkPackageToCache(packagePath, cachedPackagePath))
+    }
   }
 
   # failed to insert package into cache -- clean up and return error
-  if (!file.rename(backupPackagePath, cachedPackagePath))
+  if (!file.rename(backupPackagePath, cachedPackagePath)) {
     stop("failed to restore package '", packageName, "' in cache; package may be lost from cache")
+  }
 
   # return failure
   stop("failed to copy package '", packageName, "' to cache")
 }
 
-moveInstalledPackagesToCache <- function(project = NULL) {
-  project <- getProjectDir(project)
-
-  # Only do this is we're actually using the packrat cache
-  if (!isUsingCache(project)) return(invisible())
-
-  if (!file.exists(cacheLibDir()))
-    dir.create(cacheLibDir(), recursive = TRUE)
-
-  ## All directories within the 'lib' directory which are not symlinks are fresh
-  ## and may need to be moved
-  installedPkgPaths <- list.files(libDir(project), full.names = TRUE)
-  if (!length(installedPkgPaths)) return(invisible())
-
-  needsMove <- installedPkgPaths[sapply(installedPkgPaths, Negate(is.symlink))]
-
-  if (isVerboseCache()) {
-    message("Promoting newly installed packages to the package cache.")
-  }
-
-  # for each package installed that is not a symlink, we migrate it to the cache
-  for (package in needsMove) {
-    # copy package into cache
-    cachedPackagePath <- moveInstalledPackageToCache(package)
-    symlink(normalizePath(cachedPackagePath, winslash = "/", mustWork = TRUE), package)
-  }
-
-}
-
 # Pull out cached package information from the DESCRIPTION
 cachedPackages <- function(cacheDir = cacheLibDir(), fields = NULL) {
 
