@@ -35,10 +35,11 @@
 #' @keywords internal
 appDependencies <- function(project = NULL,
                             available.packages = NULL,
-                            fields = c("Imports", "Depends", "LinkingTo"),
+                            fields = opts$snapshot.fields(),
                             implicit.packrat.dependency = TRUE) {
 
-  if (is.null(available.packages)) available.packages <- available.packages()
+  if (is.null(available.packages))
+    available.packages <- availablePackages()
 
   project <- getProjectDir(project)
 
@@ -95,7 +96,12 @@ appDependencies <- function(project = NULL,
   if (is.null(result))
     return(character())
 
-  sort_c(result)
+  sorted <- sort_c(result)
+
+  # some users have seen empty package names discovered here
+  # although we don't know the underlying cause, we should
+  # just filter these out as we know they can't be valid
+  setdiff(sorted, "")
 }
 
 # detect all package dependencies for a directory of files
@@ -103,7 +109,7 @@ dirDependencies <- function(dir) {
   dir <- normalizePath(dir, winslash = '/')
 
   # first get the packages referred to in source code
-  pattern <- "\\.[rR]$|\\.[rR]md$|\\.[rR]nw$|\\.[rR]pres$"
+  pattern <- "[.](?:r|rmd|rnw|rpres)$"
   pkgs <- character()
   R_files <- list.files(dir,
                         pattern = pattern,
@@ -113,9 +119,30 @@ dirDependencies <- function(dir) {
 
   ## Avoid anything within the packrat directory itself -- all inference
   ## should be done on user code
-  packratDirRegex <- paste("^", "packrat", sep = "")
+  packratDirRegex <- "(?:^|/)packrat"
   R_files <- grep(packratDirRegex, R_files, invert = TRUE, value = TRUE)
 
+  ## Avoid anything on the list of ignored directories
+  ignoredDir <- get_opts("ignored.directories")
+  if (length(ignoredDir) > 0) {
+    # Make sure all the directories end with a slash...
+    ignoredDir <- ifelse(
+      substr(ignoredDir, nchar(ignoredDir), nchar(ignoredDir)) != "/",
+      paste0(ignoredDir, "/"),
+      ignoredDir
+    )
+
+    # Make a regex to match any of them.
+    ignoredDirRegex <- paste0(
+      "(?:^",
+      paste0(
+        ignoredDir,
+        collapse=")|(?:^"
+      ),
+      ")"
+    )
+    R_files <- grep(ignoredDirRegex, R_files, invert = TRUE, value = TRUE)
+  }
 
   sapply(R_files, function(file) {
     filePath <- file.path(dir, file)
@@ -157,13 +184,66 @@ yamlDeps <- function(yaml) {
   )
 }
 
+stripAltEngines <- function(file, encoding) {
+  contents <- readLines(file, encoding = encoding)
+
+  # generate a list of all the headers
+  engineHeaders <- which(grepl("^## --.*engine=", contents))
+  allHeaders <- c(which(grepl("^## --", contents)), length(contents))
+
+  # calculate the end of each alternate engine code block (the beginning of the
+  # very next code block)
+  engineEnds <- vapply(engineHeaders, function(x) {
+    allHeaders[min(which(allHeaders > x))] - 1
+  }, 0)
+
+  # exclude the alternate engine code block lines
+  regions <- rep.int(TRUE, length(contents))
+  for (h in seq_along(engineHeaders)) {
+    regions[engineHeaders[[h]]:engineEnds[[h]]] <- FALSE
+  }
+
+  writeLines(contents[regions], file)
+}
+
 fileDependencies.Rmd <- function(file) {
 
   deps <- "rmarkdown"
 
+  # try using an evaluate-based approach for dependencies
+  if (knitrHasEvaluateHook()) {
+
+    # attempt to load rmarkdown
+    isRmarkdownLoaded <- "rmarkdown" %in% loadedNamespaces()
+    if (requireNamespace("rmarkdown", quietly = TRUE)) {
+
+      # unload rmarkdown after we're done with it if it
+      # wasn't already loaded
+      if (!isRmarkdownLoaded) {
+        on.exit(
+          try(unloadNamespace("rmarkdown"), silent = TRUE),
+          add = TRUE
+        )
+      }
+
+      # render with a custom evaluate hook to discover dependencies
+      deps <- c(deps, fileDependencies.Rmd.evaluate(file))
+    }
+  }
+
+  # we don't know this file's encoding, so presume the default encoding
+  encoding <- getOption("encoding")
+  format <- NULL
+
   # check whether the default output format references a package
   if (requireNamespace("rmarkdown", quietly = TRUE)) {
-    format <- rmarkdown::default_output_format(file)
+    tryCatch({
+      format <- rmarkdown::default_output_format(file)
+    }, error = function(e) {
+      # if we can't parse the YAML header with the default encoding, try UTF-8
+      encoding <<- "UTF-8"
+      format <<- rmarkdown::default_output_format(file, encoding)
+    })
     components <- strsplit(format$name, "::")[[1]]
     if (length(components) == 2) {
       deps <- c(deps, components[[1]])
@@ -172,7 +252,7 @@ fileDependencies.Rmd <- function(file) {
 
   # We need to check for and parse YAML frontmatter if necessary
   yamlDeps <- NULL
-  content <- readLines(file)
+  content <- readLines(file, encoding = encoding, warn = FALSE)
   if (hasYamlFrontMatter(content)) {
 
     # Extract the YAML frontmatter.
@@ -186,20 +266,24 @@ fileDependencies.Rmd <- function(file) {
     deps <- c(deps, yamlDeps)
 
     # Extract additional dependencies from YAML parameters.
-    if (requireNamespace("knitr", quietly = TRUE) &&
-        packageVersion("knitr") >= "1.10.18") {
+    if (packageVersion("knitr") >= "1.10.18" &&
+        requireNamespace("knitr", quietly = TRUE))
+    {
+      # attempt to extract knitr params from yaml
+      knitParams <- tryCatch(
+        knitr::knit_params_yaml(yaml, evaluate = FALSE),
+        error = function(e) {
+          warning(e)
+          NULL
+        }
+      )
 
-      knitParams <- knitr::knit_params_yaml(yaml, evaluate = FALSE)
-      if (length(knitParams) > 0) {
+      if (length(knitParams)) {
         deps <- c(deps, "shiny")
         for (param in knitParams) {
           if (!is.null(param$expr)) {
-            parsed <- tryCatch(
-              parse(text = param$expr),
-              error = function(e) NULL
-            )
-
-            if (length(parsed))
+            parsed <- quietly(parse(text = param$expr))
+            if (!inherits(parsed, "error"))
               deps <- c(deps, expressionDependencies(parsed))
           }
         }
@@ -208,7 +292,6 @@ fileDependencies.Rmd <- function(file) {
     }
   }
 
-
   # Escape hatch for empty .Rmd files
   if (!length(content) || identical(unique(gsub("[[:space:]]", "", content, perl = TRUE)), "")) {
     return(deps)
@@ -223,17 +306,18 @@ fileDependencies.Rmd <- function(file) {
     if (!knitrIsLoaded && "knitr" %in% loadedNamespaces()) {
       try(unloadNamespace("knitr"), silent = TRUE)
     }
-  })
+  }, add = TRUE)
 
   if (requireNamespace("knitr", quietly = TRUE)) {
     tempfile <- tempfile()
     on.exit(unlink(tempfile))
     tryCatch(silent(
-      knitr::knit(file, output = tempfile, tangle = TRUE)
+      knitr::knit(file, output = tempfile, tangle = TRUE, encoding = encoding)
     ), error = function(e) {
       message("Unable to tangle file '", file, "'; cannot parse dependencies")
       character()
     })
+    stripAltEngines(tempfile, encoding)
     c(deps, fileDependencies.R(tempfile))
   } else {
     warning("knitr is required to parse dependencies but is not available")
@@ -270,21 +354,25 @@ fileDependencies.R <- function(file) {
   # build a list of package dependencies to return
   pkgs <- character()
 
-  # parse file and examine expressions
-  tryCatch({
-    # parse() generates a warning when the file has an incomplete last line, but
-    # it still parses the file correctly; ignore this and other warnings.
-    # We'll still halt when parsing fails.
-    exprs <- suppressWarnings(parse(file, n = -1L))
-    for (i in seq_along(exprs))
-      pkgs <- append(pkgs, expressionDependencies(exprs[[i]]))
-  }, error = function(e) {
+  # parse file and examine expressions -- first attempt to
+  # parse in system encoding, then try again with UTF-8
+  exprs <- quietly(parse(file, n = -1L))
+  if (inherits(exprs, "error"))
+    exprs <- quietly(parse(file, n = -1L, encoding = "UTF-8"))
+
+  # report parse errors to the user
+  if (inherits(exprs, "error")) {
     warning(paste("Failed to parse", file, "; dependencies in this file will",
                   "not be discovered."))
-  })
+    exprs <- NULL
+  }
+
+  # extract expression dependencies
+  for (i in seq_along(exprs))
+    pkgs <- append(pkgs, expressionDependencies(exprs[[i]]))
 
   # return packages
-  unique(pkgs)
+  setdiff(unique(pkgs), "")
 }
 
 anyOf <- function(object, ...) {
@@ -462,3 +550,74 @@ isShinyApp <- function(project) {
 
   return(FALSE)
 }
+
+knitrHasEvaluateHook <- function() {
+  isKnitrLoaded <- "knitr" %in% loadedNamespaces()
+  if (!requireNamespace("knitr", quietly = TRUE))
+    return(FALSE)
+
+  if (!isKnitrLoaded) {
+    on.exit(
+      try(unloadNamespace("knitr"), silent = TRUE),
+      add = TRUE
+    )
+  }
+
+  hooks <- knitr::knit_hooks$get()
+  "evaluate" %in% names(hooks)
+}
+
+
+fileDependencies.Rmd.evaluate <- function(file) {
+
+  # discovered packages (to be updated by evaluate hook)
+  deps <- list()
+
+  # save old hook and install our custom hook
+  evaluate_hook <- knitr::knit_hooks$get("evaluate")
+  on.exit(knitr::knit_hooks$set(evaluate = evaluate_hook), add = TRUE)
+  knitr::knit_hooks$set(evaluate = function(code, ...) {
+    try(silent = TRUE, {
+      parsed <- parse(text = code, encoding = "UTF-8")
+      deps <<- c(deps, expressionDependencies(parsed))
+    })
+  })
+
+  # rudely override knitr's 'inline_exec' function so
+  # that we can detect dependencies within inline chunks
+  knitr <- asNamespace("knitr")
+  if (exists("inline_exec", envir = knitr)) {
+
+    inline_exec <- yoink("knitr", "inline_exec")
+    do.call("unlockBinding", list("inline_exec", knitr))
+    assign("inline_exec", function(block, ...) {
+
+      # do our own special stuff
+      try(silent = TRUE, {
+        code <- paste(block$code, collapse = "\n")
+        parsed <- parse(text = code, encoding = "UTF-8")
+        deps <<- c(deps, expressionDependencies(parsed))
+      })
+
+      # return block input without evaluating anything
+      block$input
+
+    }, envir = knitr)
+
+    on.exit({
+      assign("inline_exec", inline_exec, envir = knitr)
+      do.call("lockBinding", list("inline_exec", knitr))
+    }, add = TRUE)
+
+  }
+
+  # attempt to render document with our custom hook active
+  outfile <- tempfile()
+  tryCatch(
+    rmarkdown::render(file, output_file = outfile, quiet = TRUE),
+    error = identity
+  )
+  unlink(outfile)
+
+  unique(unlist(deps, recursive = TRUE))
+}
