@@ -1,16 +1,16 @@
 
-# check whether the specified file ends with newline
-ends_with_newline <- function(path) {
-  conn <- file(path, open = "rb", raw = TRUE)
-  on.exit(close(conn))
-  seek(conn, where = -1, origin = "end")
-  lastByte <- readBin(conn, "raw", n = 1)
-  lastByte == 0x0a
+# ensure that a file ends with a single newline
+normalizeDcf <- function(path) {
+  n <- file.info(path)$size
+  contents <- readChar(path, n, TRUE)
+  replaced <- gsub("\n*$", "", contents)
+  if (!identical(contents, replaced))
+    cat(replaced, file = path, sep = "\n")
+  path
 }
 
 appendToDcf <- function(path, records) {
-  if (!ends_with_newline(path))
-    cat('\n', sep = '', file = path, append = TRUE)
+  normalizeDcf(path)
   write_dcf(records, path, append = TRUE)
 }
 
